package com.travel.erp.controller;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.model.LeadSource;
import com.travel.erp.model.LeadStatus;
import com.travel.erp.service.LeadService;
import com.travel.erp.view.model.Failure;
import com.travel.erp.view.model.LeadModel;
import com.travel.erp.view.model.Success;

@RestController
public class LeadController extends BaseController {

	private static final Logger logger = LoggerFactory
			.getLogger(LeadController.class);

	@Autowired
	private LeadService leadService;

	@RequestMapping(value = "/lead", method = RequestMethod.POST, produces = "application/json; charset=utf-8")
	public @ResponseBody Object addLead(@RequestBody LeadModel leadModel) {
		try {
			int id = leadService.createLead(leadModel);
			if (id > 0) {
				Success success = new Success(id, "Lead added successfully");
				return success;
			} else {
				throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
						"Unknown error occurred");
			}
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/lead/{id}", method = RequestMethod.PUT, produces = "application/json; charset=utf-8")
	public @ResponseBody Object updateLead(@PathVariable int id,
			@RequestBody LeadModel leadModel) {
		try {
			leadModel.setErp_id(id);
			leadService.updateLead(leadModel);
			Success success = new Success(id, "Lead updated successfully");
			return success;

		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/lead/{id}", method = RequestMethod.DELETE)
	public @ResponseBody Object deleteLead(@PathVariable int id) {
		try {
			leadService.deleteLead(id);
			Success success = new Success(id, "Lead deleted successfully");
			return success;
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/lead/{id}", method = RequestMethod.GET, produces = "application/json; charset=utf-8")
	public @ResponseBody Object getLeadInfo(@PathVariable int id) {
		try {
			LeadModel lead = leadService.getLeadInfo(id);
			if (lead != null) {
				return lead;
			} else {
				throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
						"Unknown error occurred");
			}
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/lead", method = RequestMethod.GET)
	public @ResponseBody Object getLeads(
			@RequestParam(required = false) Integer leadId,
			@RequestParam(required = false) String emailId,
			@RequestParam(required = false) String phoneNo,
			@RequestParam(required = false) String destinations,
			@RequestParam(required = false) Integer customerId,
			@RequestParam(required = false) String departureDate,
			@RequestParam(required = false) String leadSource,
			@RequestParam(required = false) String leadStatus,
			@RequestParam(required = false) Integer createdBy,
			@RequestParam(required = false) Integer assignedTo,
			@RequestParam(required = false) String fromDate,
			@RequestParam(required = false) String toDate) {
		try {
			return leadService.searchLeads(leadId, emailId, phoneNo,
					destinations, customerId, departureDate, leadSource,
					leadStatus, createdBy, assignedTo, fromDate, toDate);
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}

	}
	
	
	@RequestMapping(value = "/leadSources", method = RequestMethod.GET)
	public @ResponseBody Object getLeadSources() {
		try {
			List<String> leadSourcesValues = new ArrayList<>();
			List<LeadSource> leadSources = leadService.getLeadSources();
			for (LeadSource source : leadSources) {
				leadSourcesValues.add(source.getLeadSource());
			}
			return leadSourcesValues;
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/leadStatusValues", method = RequestMethod.GET)
	public @ResponseBody Object getLeadStatusValues() {
		try {
			List<String> leadStatusValues = new ArrayList<>();
			List<LeadStatus> leadStatuses = leadService.getLeadStatusValues();
			for (LeadStatus status : leadStatuses) {
				leadStatusValues.add(status.getStatus());
			}
			return leadStatusValues;
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}
}
