package com.travel.erp.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.service.NotesService;
import com.travel.erp.view.model.Failure;
import com.travel.erp.view.model.NoteModel;
import com.travel.erp.view.model.Success;

@RestController
public class NotesController extends BaseController {

	private static final Logger logger = LoggerFactory
			.getLogger(NotesController.class);

	@Autowired
	private NotesService notesService;

	@RequestMapping(path = "/lead/{leadId}/note", method = RequestMethod.POST)
	public @ResponseBody Object addLeadNote(@RequestBody NoteModel noteModel,
			@PathVariable int leadId) {
		try {
			noteModel.setErp_leadId(leadId);
			Integer noteId = notesService.addNotesToLead(noteModel);

			if (noteId != null) {
				return new Success(noteId, "Note added succesfully");
			} else {
				throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
						"Unknown error occurred");
			}
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}

	}

	@RequestMapping(value = "/lead/{leadId}/notes", method = RequestMethod.GET)
	public @ResponseBody Object getNotesOfALead(@PathVariable int leadId) {
		try {
			return notesService.getNotesOfALead(leadId);
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/booking/{bookingId}/notes", method = RequestMethod.GET)
	public @ResponseBody Object getNotesOfABooking(@PathVariable int bookingId) {
		try {
			return notesService.getNotesOfABooking(bookingId);
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/lead/{leadId}/note/{noteId}", method = RequestMethod.GET)
	public @ResponseBody Object getNoteInfo(@PathVariable int leadId,
			@PathVariable int noteId) {
		try {
			return notesService.getNoteInfo(noteId);
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/lead/{leadId}/note/{noteId}", method = RequestMethod.PUT)
	public @ResponseBody Object updateLeadNote(@PathVariable int leadId,
			@PathVariable int noteId, @RequestBody NoteModel noteModel) {
		try {
			noteModel.setErp_leadId(leadId);
			noteModel.setErp_noteId(noteId);
			notesService.updateNoteOfALead(leadId, noteModel);
			return new Success(noteId, "Note updated successfully");
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/booking/{bookingId}/note/{noteId}", method = RequestMethod.PUT)
	public @ResponseBody Object updateBookingNote(@PathVariable int bookingId,
			@PathVariable int noteId, @RequestBody NoteModel noteModel) {
		try {
			noteModel.setErp_noteId(noteId);
			notesService.updateNoteOfABooking(bookingId, noteModel);
			return new Success(noteId, "Note updated successfully");
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}

	@RequestMapping(value = "/lead/{leadId}/note/{noteId}", method = RequestMethod.DELETE)
	public @ResponseBody Object deleteNote(@PathVariable int leadId,
			@PathVariable int noteId) {
		try {
			notesService.deleteNote(noteId);
			return new Success(noteId, "Note deleted successfully");
		} catch (ERPException e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(e.getCode(), e.getMessage());
			return failure;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			Failure failure = new Failure(ErrorCodes.UNKNOWN_ERROR,
					e.getMessage());
			return failure;
		}
	}
}
