package com.travel.erp.dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.hibernate.exception.ConstraintViolationException;
import org.hibernate.exception.DataException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.travel.erp.dao.AlertsDao;
import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.model.Alert;
import com.travel.erp.model.Employee;

@Repository("alertsDao")
public class AlertsDaoImpl implements AlertsDao {

	@Autowired
	private SessionFactory sessionFactory;

	public AlertsDaoImpl() {
	}

	public AlertsDaoImpl(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Override
	@Transactional
	public int createLeadAlert(Alert alert) {
		try {
			Date date = new Date();
			alert.setUpdatedAt(date);
			alert.setCreatedAt(date);
			sessionFactory.getCurrentSession().save(alert);
			sessionFactory.getCurrentSession().flush();
			return alert.getAlertId();
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("leadId")) {
					throw new ERPException(ErrorCodes.INVALID_LEAD_ID,
							"Invalid lead id provided", e);
				}
				if (message.contains("alertAssignee")) {
					throw new ERPException(
							ErrorCodes.INVALID_ASSIGNEE_ID_FOR_LEAD,
							"Invalid assignee id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}

			}
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_ALERTS, e
					.getSQLException().getMessage(), e);

		} catch (DataException e) {
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_ALERTS,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error occurred while adding notes", e);
		}
	}

	@Override
	@Transactional
	public void updateAlert(Alert alert) {
		try {
			Session session = sessionFactory.getCurrentSession();
			Alert oldAlert = (Alert) session.get(Alert.class,
					alert.getAlertId());
			if (oldAlert == null) {
				throw new ERPException(ErrorCodes.COULD_NOT_FIND_ALERT,
						"Couldn't find alert with Id : " + alert.getAlertId());
			}
			oldAlert.setAlertNotes(StringUtils.isEmpty(alert.getAlertNotes()) ? oldAlert
					.getAlertNotes() : alert.getAlertNotes());

			oldAlert.setAlertStatus(StringUtils.isEmpty(alert.getAlertStatus()) ? oldAlert
					.getAlertStatus() : alert.getAlertStatus());
			oldAlert.setAlertDate(alert.getAlertDate() == null ? oldAlert
					.getAlertDate() : alert.getAlertDate());
			oldAlert.setAlertAssignee(alert.getAlertAssignee() == null ? oldAlert
					.getAlertAssignee() : alert.getAlertAssignee());
			oldAlert.setEmailNotification(alert.isEmailNotification() == null ? oldAlert
					.isEmailNotification() : alert.isEmailNotification());
			oldAlert.setUpdatedAt(new Date());
			session.update(oldAlert);
			session.flush();

		} catch (ERPException e) {
			throw e;
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("leadId")) {
					throw new ERPException(ErrorCodes.INVALID_LEAD_ID,
							"Invalid lead id provided", e);
				}
				if (message.contains("alertAssignee")) {
					throw new ERPException(
							ErrorCodes.INVALID_ASSIGNEE_ID_FOR_LEAD,
							"Invalid assignee id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}

			}
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_ALERTS, e
					.getSQLException().getMessage(), e);
		} catch (DataException e) {
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_ALERTS,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_UPDATING_ALERT,
					"Error occurred while updating lead", e);
		}
	}

	@Override
	@Transactional
	public void deleteAlert(int alertId) {
		try {
			Session session = sessionFactory.getCurrentSession();
			Alert alert = (Alert) session.get(Alert.class, alertId);
			if (alert == null) {
				throw new ERPException(ErrorCodes.COULD_NOT_FIND_ALERT,
						"Couldn't find alert with Id : " + alertId);
			}

			session.delete(alert);
			session.flush();
		} catch (ERPException e) {
			throw e;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_DELETING_ALERT,
					"Error occurred while deleting alert", e);
		}

	}

	@Override
	public Alert getAlertDetails(int alertId) {
		Session session = sessionFactory.openSession();
		try {
			return (Alert) session.get(Alert.class, alertId);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_ALERT,
					"Couldn't find alert with alertId : " + alertId, e);
		} finally {
			session.close();
		}
	}

	@Override
	public List<Alert> getAlertsOfLead(int leadId, String alertStatus) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(Alert.class);
			query.add(Restrictions.eq("leadId", leadId));
			if (StringUtils.hasText(alertStatus)
					&& !alertStatus.equalsIgnoreCase("ALL")) {
				query.add(Restrictions.ilike("alertStatus", alertStatus));
			}
			List<Alert> list = (List<Alert>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error while getting alerts for leadId : " + leadId);
		} finally {
			session.close();
		}
	}

	@Override
	public List<Alert> getAlertsByAssignedTo(int employeeId, String alertStatus) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(Alert.class);
			Employee employee = new Employee();
			employee.setId(employeeId);
			query.add(Restrictions.eq("alertAssignee", employee));
			if (StringUtils.hasText(alertStatus)
					&& !alertStatus.equalsIgnoreCase("ALL")) {
				query.add(Restrictions.ilike("alertStatus", alertStatus));
			}
			List<Alert> list = (List<Alert>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error while getting alerts for assignee : " + employeeId);
		} finally {
			session.close();
		}
	}

	@Override
	public List<Alert> getAlertsByAlertDate(Date date) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(Alert.class);
			query.add(Restrictions.le("alertDate", date));
			query.add(Restrictions.ilike("alertStatus", "ACTIVE"));
			List<Alert> list = (List<Alert>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error while getting alerts for date : " + date.toString());
		} finally {
			session.close();
		}
	}

}
