/**
 * 
 */
package com.travel.erp.dao.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.travel.erp.dao.BookingDao;
import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.model.Booking;

/**
 * @author Himanshu
 *
 */
@Repository("bookingDao")
public class BookingDaoImpl implements BookingDao {

	private static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

	private static final Logger logger = LoggerFactory.getLogger(BookingDaoImpl.class);

	@Autowired
	private SessionFactory sessionFactory;

	public BookingDaoImpl() {
	}

	public BookingDaoImpl(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Override
	public Booking getBooking(int bookingId) {
		Session session = sessionFactory.openSession();
		try {
			return (Booking) session.get(Booking.class, bookingId);
		} catch (HibernateException exception) {
			logger.error(exception.toString());
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_BOOKING,
					"Booking lookup failed for booking Id: [" + bookingId + "]");
		} finally {
			session.flush();
			session.close();
		}
	}

	@Override
	@Transactional
	public int saveBooking(Booking booking) {
		Session session = sessionFactory.openSession();
		try {
			return (int) session.save(booking);
		} catch (HibernateException ex) {
			ex.printStackTrace();
			throw new ERPException(ErrorCodes.COULD_NOT_SAVE_BOOKING, "Attempt to save booking failed");
		} finally {
			session.flush();
			session.close();
		}
	}

	@Override
	@Transactional
	public void updateBooking(Booking booking) {
		Session session = sessionFactory.openSession();
		Booking existingBooking = getBooking(booking.getBookingId());
		if (existingBooking != null) {
			logger.debug("existing booking found");
//			Hibernate.initialize(existingBooking.getHotelBookings());
			mergeBooking(booking, existingBooking);
			try {
				session.update(booking);
			} catch (HibernateException ex) {
				ex.printStackTrace();
				throw new ERPException(ErrorCodes.BOOKING_CANNOT_BE_UPDATED,
						"Booking with Id [" + booking.getBookingId() + "] can't be updated since it doesnot exist");
			} finally {
				session.flush();
				session.close();
			}
		} else {
			logger.debug("existing booking not found");
			throw new ERPException(ErrorCodes.BOOKING_CANNOT_BE_UPDATED,
					"Booking with Id [" + booking.getBookingId() + "] can't be updated since it doesnot exist");
		}
	}

	private void mergeBooking(Booking booking, Booking existingBooking) {
		logger.debug("merging bookings");
		if (booking.getBookingName() == null || "".equals(booking.getBookingName())) {
			booking.setBookingName(existingBooking.getBookingName());
		}

		if (booking.getPackageCode() == null || "".equals(booking.getPackageCode())) {
			booking.setPackageCode(existingBooking.getPackageCode());
		}

		if (booking.getSalesPerson() == null) {
			booking.setSalesPerson(existingBooking.getSalesPerson());
		}

		if (booking.getRoomCount() == 0) {
			booking.setSalesPerson(existingBooking.getSalesPerson());
		}

		if (booking.getPickupLocation() == null || "".equals(booking.getPickupLocation())) {
			booking.setPickupLocation(existingBooking.getPickupLocation());
		}

		if (booking.getDropLocation() == null || "".equals(booking.getDropLocation())) {
			booking.setDropLocation(existingBooking.getDropLocation());
		}

		if (booking.getVehicle() == null || "".equals(booking.getVehicle())) {
			booking.setVehicle(existingBooking.getVehicle());
		}

		if (booking.getVehicleCost() == null || "".equals(booking.getVehicleCost())) {
			booking.setVehicleCost(existingBooking.getVehicleCost());
		}

		if (booking.getProposedPackageCost() == null || "".equals(booking.getProposedPackageCost())) {
			booking.setProposedPackageCost(existingBooking.getProposedPackageCost());
		}

		if (!booking.isTaxIncluded()) {
			booking.setTaxIncluded(existingBooking.isTaxIncluded());
		}

		if (booking.getBookingStatus() == null || "".equals(booking.getBookingStatus())) {
			booking.setBookingStatus(existingBooking.getBookingStatus());
		}

		if (booking.getInclusions() == null || "".equals(booking.getInclusions())) {
			booking.setInclusions(existingBooking.getInclusions());
		}

		if (booking.getExclusions() == null || "".equals(booking.getExclusions())) {
			booking.setExclusions(existingBooking.getExclusions());
		}

		if (booking.getCreatedBy() == 0) {
			booking.setCreatedBy(existingBooking.getCreatedBy());
		}

		if (booking.getHotelBookings() != null) {
			booking.getHotelBookings().addAll(existingBooking.getHotelBookings());
		} else {
			booking.setHotelBookings(existingBooking.getHotelBookings());
		}

		if (booking.getTravelBookings() != null) {
			booking.getTravelBookings().addAll(existingBooking.getTravelBookings());
		} else {
			booking.setTravelBookings(existingBooking.getTravelBookings());
		}

		booking.setUpdatedAt(new SimpleDateFormat(DATE_FORMAT).format(new Date()));
	}

	@Override
	@Transactional
	public void deleteBooking(int bookingId) {
		Session session = sessionFactory.openSession();
		Booking existingBooking = getBooking(bookingId);
		if (existingBooking != null) {
			try {
				session.delete(existingBooking);
			} catch (HibernateException ex) {
				ex.printStackTrace();
				throw new ERPException(ErrorCodes.BOOKING_CANNOT_BE_DELETED,
						"Booking with Id [" + bookingId + "] can't be deleted since it doesnot exist");
			} finally {
				session.flush();
				session.close();
			}
		} else {
			throw new ERPException(ErrorCodes.BOOKING_CANNOT_BE_DELETED,
					"Booking with Id [" + bookingId + "] can't be deleted since it doesnot exist");
		}
	}

	@Override
	public List<Booking> getAllBookings() {
		Session session = sessionFactory.openSession();
		try {
			List<Booking> bookingList = new ArrayList<>();
			List<?> list = session.createQuery("FROM Booking").list();
			for (Object object : list) {
				Booking booking = (Booking) object;
				Hibernate.initialize(booking);
				Hibernate.initialize(booking.getHotelBookings());
				bookingList.add(booking);
			}
			return bookingList;
		} catch (HibernateException ex) {
			ex.printStackTrace();
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_BOOKING, "Error while getting Bookings");
		} finally {
			session.flush();
			session.close();
		}
	}

	@Override
	public List<Booking> getBookingBySalesPersonId(int salesPersonId) {
		Session session = sessionFactory.openSession();
		try {
			Criteria criteria = session.createCriteria(Booking.class).createAlias("salesPerson", "sp")
					.add(Restrictions.eq("sp.id", salesPersonId));

			List<Booking> bookingList = new ArrayList<>();
			List<?> list = criteria.list();
			for (Object object : list) {
				Booking booking = (Booking) object;
				Hibernate.initialize(booking);
				Hibernate.initialize(booking.getHotelBookings());
				bookingList.add(booking);
			}
			return bookingList;
		} catch (HibernateException ex) {
			ex.printStackTrace();
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_BOOKING,
					"Error while getting Bookings for Sales person with Id [" + salesPersonId + "]");
		} finally {
			session.flush();
			session.close();
		}
	}

	@Override
	public List<Booking> getBookingByStatus(String bookingStatus) {
		Session session = sessionFactory.openSession();
		try {
			Criteria criteria = session.createCriteria(Booking.class)
					.add(Restrictions.eq("bookingStatus", bookingStatus));

			List<Booking> bookingList = new ArrayList<>();
			List<?> list = criteria.list();
			for (Object object : list) {
				Booking booking = (Booking) object;
				Hibernate.initialize(booking);
				Hibernate.initialize(booking.getHotelBookings());
				bookingList.add(booking);
			}
			return bookingList;
		} catch (HibernateException ex) {
			ex.printStackTrace();
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_BOOKING,
					"Error while getting Bookings with Status [" + bookingStatus + "]");
		} finally {
			session.flush();
			session.close();
		}
	}

	@Override
	public List<Booking> getBookingByPackageCode(String packageCode) {
		Session session = sessionFactory.openSession();
		try {
			Criteria criteria = session.createCriteria(Booking.class)
					.add(Restrictions.ilike("packageCode", "%" + packageCode + "%"));

			List<Booking> bookingList = new ArrayList<>();
			List<?> list = criteria.list();
			for (Object object : list) {
				Booking booking = (Booking) object;
				Hibernate.initialize(booking);
				Hibernate.initialize(booking.getHotelBookings());
				bookingList.add(booking);
			}
			return bookingList;
		} catch (HibernateException ex) {
			ex.printStackTrace();
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_BOOKING,
					"Error while getting Bookings with Package code [" + packageCode + "]");
		} finally {
			session.flush();
			session.close();
		}
	}

	@Override
	public List<Booking> getBookingByBookingName(String bookingName) {
		Session session = sessionFactory.openSession();
		try {
			Criteria criteria = session.createCriteria(Booking.class)
					.add(Restrictions.ilike("bookingName", bookingName));

			List<Booking> bookingList = new ArrayList<>();
			List<?> list = criteria.list();
			for (Object object : list) {
				Booking booking = (Booking) object;
				Hibernate.initialize(booking);
				Hibernate.initialize(booking.getHotelBookings());
				bookingList.add(booking);
			}
			return bookingList;
		} catch (HibernateException ex) {
			ex.printStackTrace();
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_BOOKING,
					"Error while getting Bookings with Booking name [" + bookingName + "]");
		} finally {
			session.flush();
			session.close();
		}
	}

	@Override
	public List<Booking> getBookingByCreatedById(int createdById) {
		Session session = sessionFactory.openSession();
		try {
			Criteria criteria = session.createCriteria(Booking.class).add(Restrictions.eq("createdBy", createdById));

			List<Booking> bookingList = new ArrayList<>();
			List<?> list = criteria.list();
			for (Object object : list) {
				Booking booking = (Booking) object;
				Hibernate.initialize(booking);
				Hibernate.initialize(booking.getHotelBookings());
				bookingList.add(booking);
			}
			return bookingList;
		} catch (HibernateException ex) {
			ex.printStackTrace();
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_BOOKING,
					"Error while getting Bookings with Created By Id [" + createdById + "]");
		} finally {
			session.flush();
			session.close();
		}
	}
}