package com.travel.erp.dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;
import org.hibernate.exception.ConstraintViolationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.travel.erp.dao.CustomerDao;
import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.model.Customer;
import com.travel.erp.model.Employee;

@Repository("customerDao")
public class CustomerDaoImpl implements CustomerDao {

	@Autowired
	private SessionFactory sessionFactory;

	public CustomerDaoImpl() {
	}

	public CustomerDaoImpl(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Transactional
	public int createCustomer(Customer customer) {
		try {
			sessionFactory.getCurrentSession().saveOrUpdate(customer);
			sessionFactory.getCurrentSession().flush();
			return customer.getId();
		} catch (ConstraintViolationException e) {
			if (e.getSQLException().getMessage().contains("emailId_UNIQUE")) {
				throw new ERPException(ErrorCodes.DUPLICATE_CUSTOMER_EMAIL_ID,
						"Duplicate email id", e);
			} else if (e.getSQLException().getMessage()
					.contains("phoneNo_UNIQUE")) {
				throw new ERPException(ErrorCodes.DUPLICATE_CUSTOMER_PHONE,
						"Duplicate phone number	", e);
			}
		}
		return -1;
	}

	@Override
	@Transactional
	public Integer editCustomer(Customer customer) {
		try {
			Session session = sessionFactory.getCurrentSession();
			Customer oldCustomer = (Customer) session.get(Customer.class,
					customer.getId());
			if (oldCustomer == null) {
				throw new ERPException(ErrorCodes.COULD_NOT_FIND_CUSTOMER,
						"Couldn't find user with userId : " + customer.getId());
			}
			oldCustomer
					.setCity(StringUtils.isEmpty(customer.getCity()) ? oldCustomer
							.getCity() : customer.getCity());
			oldCustomer
					.setCountry(StringUtils.isEmpty(customer.getCountry()) ? oldCustomer
							.getCountry() : customer.getCountry());
			oldCustomer
					.setDateOfBirth(customer.getDateOfBirth() == null ? oldCustomer
							.getDateOfBirth() : customer.getDateOfBirth());
			oldCustomer
					.setEmailId(StringUtils.isEmpty(customer.getEmailId()) ? oldCustomer
							.getEmailId() : customer.getEmailId());
			oldCustomer
					.setPhoneNo(StringUtils.isEmpty(customer.getPhoneNo()) ? oldCustomer
							.getPhoneNo() : customer.getPhoneNo());
			oldCustomer
					.setName(StringUtils.isEmpty(customer.getName()) ? oldCustomer
							.getName() : customer.getName());
			oldCustomer.setUpdateTime(new Date());
			sessionFactory.getCurrentSession().update(oldCustomer);
			sessionFactory.getCurrentSession().flush();
			return customer.getId();
		} catch (ConstraintViolationException e) {
			if (e.getSQLException().getMessage().contains("emailId_UNIQUE")) {
				throw new ERPException(ErrorCodes.DUPLICATE_CUSTOMER_EMAIL_ID,
						"Duplicate email id", e);
			} else if (e.getSQLException().getMessage()
					.contains("phoneNo_UNIQUE")) {
				throw new ERPException(ErrorCodes.DUPLICATE_CUSTOMER_PHONE,
						"Duplicate phone number	", e);
			}
		}
		return null;
	}

	public Customer getCustomer(int id) {
		Session session = sessionFactory.openSession();
		try {
			return (Customer) session.get(Customer.class, id);

		} catch (Exception e) {
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_CUSTOMER,
					"Couldn't find user with userId : " + id, e);
		} finally {
			session.close();
		}

	}

	public List<Customer> searchCustomerByEmailId(String emailId) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(Customer.class);
			query.add(Restrictions.like("emailId", emailId, MatchMode.START));
			List<Customer> list = (List<Customer>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error while searching customers by emailId : " + emailId);
		} finally {
			session.close();
		}
	}

	@Override
	public List<Customer> searchCustomerByPhoneNo(String phoneNo) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(Customer.class);
			query.add(Restrictions.like("phoneNo", phoneNo, MatchMode.START));
			List<Customer> list = (List<Customer>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error while searching customers by phoneNo : " + phoneNo);
		} finally {
			session.close();
		}
	}

	@Override
	public List<Customer> getAllCustomers() {
		Session session = sessionFactory.openSession();
		try {
			List<Customer> list = (List<Customer>) session.createCriteria(
					Customer.class).list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Could not retrieve customer list");
		} finally {
			session.close();
		}
	}
}
