package com.travel.erp.dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.hibernate.exception.ConstraintViolationException;
import org.hibernate.exception.DataException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.travel.erp.dao.LeadAttachmentDao;
import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.model.Lead;
import com.travel.erp.model.LeadAttachment;

@Repository("leadAttachmentDao")
public class LeadAttachmentDaoImpl implements LeadAttachmentDao {

	@Autowired
	private SessionFactory sessionFactory;

	public LeadAttachmentDaoImpl() {
	}

	public LeadAttachmentDaoImpl(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Override
	@Transactional
	public LeadAttachment addLeadAttachment(LeadAttachment attachment) {
		try {
			Date date = new Date();
			attachment.setCreationTime(date);
			attachment.setUpdateTime(date);
			sessionFactory.getCurrentSession().save(attachment);
			sessionFactory.getCurrentSession().flush();
			return attachment;
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("leadId")) {
					throw new ERPException(ErrorCodes.INVALID_LEAD_ID,
							"Invalid lead id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}

			}
			throw new ERPException(
					ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD_ATTACHMENT, e
							.getSQLException().getMessage(), e);

		} catch (DataException e) {
			throw new ERPException(
					ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD_ATTACHMENT,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_ADDING_LEAD_ATTACHMENT,
					"Error occurred while adding lead attachment", e);
		}
	}

	@Override
	@Transactional
	public List<LeadAttachment> addLeadAttachments(
			List<LeadAttachment> attachments) {
		try {
			Date date = new Date();
			for (LeadAttachment attachment : attachments) {
				attachment.setCreationTime(date);
				attachment.setUpdateTime(date);
				sessionFactory.getCurrentSession().save(attachment);
				sessionFactory.getCurrentSession().flush();
				sessionFactory.getCurrentSession().clear();
			}
			return attachments;
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("leadId")) {
					throw new ERPException(ErrorCodes.INVALID_LEAD_ID,
							"Invalid lead id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}

			}
			throw new ERPException(
					ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD_ATTACHMENT, e
							.getSQLException().getMessage(), e);

		} catch (DataException e) {
			throw new ERPException(
					ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD_ATTACHMENT,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_ADDING_LEAD_ATTACHMENT,
					"Error occurred while adding lead attachment", e);
		}
	}

	@Override
	public LeadAttachment getLeadAttachmentByAttachmentId(int attId) {
		Session session = sessionFactory.openSession();
		try {
			return (LeadAttachment) session.get(LeadAttachment.class, attId);

		} catch (Exception e) {
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_LEAD_ATTACHMENT,
					"Couldn't find leadAttachment with Id : " + attId, e);
		} finally {
			session.close();
		}
	}

	@Override
	public List<LeadAttachment> getAttachmentsOfALead(int leadId) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(LeadAttachment.class);
			Lead lead = new Lead();
			lead.setId(leadId);
			query.add(Restrictions.eq("lead", lead));
			List<LeadAttachment> list = (List<LeadAttachment>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error while getting leadAttachments for leadId : "
							+ leadId);
		} finally {
			session.close();
		}
	}

	@Override
	@Transactional
	public void updateLeadAttachment(LeadAttachment attachment) {
		try {
			Session session = sessionFactory.getCurrentSession();
			LeadAttachment dbAttachment = (LeadAttachment) session.get(
					LeadAttachment.class, attachment.getAttId());
			if (dbAttachment == null) {
				throw new ERPException(
						ErrorCodes.COULD_NOT_FIND_LEAD_ATTACHMENT,
						"Couldn't find leadAttachment with Id : "
								+ attachment.getAttId());
			}
			dbAttachment.setAttachmentName(StringUtils.isEmpty(attachment
					.getAttachmentName()) ? dbAttachment.getAttachmentName()
					: attachment.getAttachmentName());
			dbAttachment.setAttachmentType(StringUtils.isEmpty(attachment
					.getAttachmentType()) ? dbAttachment.getAttachmentType()
					: attachment.getAttachmentType());
			dbAttachment
					.setAttachment((attachment.getAttachment() == null || attachment
							.getAttachment().length == 0) ? dbAttachment
							.getAttachment() : attachment.getAttachment());
			dbAttachment.setCreatedBy(attachment.getCreatedBy());
			dbAttachment.setUpdateTime(new Date());
			session.update(dbAttachment);
			session.flush();

		} catch (ERPException e) {
			throw e;
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("leadId")) {
					throw new ERPException(ErrorCodes.INVALID_LEAD_ID,
							"Invalid lead id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}

			}
			throw new ERPException(
					ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD_ATTACHMENT, e
							.getSQLException().getMessage(), e);

		} catch (DataException e) {
			throw new ERPException(
					ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD_ATTACHMENT,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_ADDING_LEAD_ATTACHMENT,
					"Error occurred while adding lead attachment", e);
		}

	}

	@Override
	@Transactional
	public void deleteLeadAttachment(int atttId) {
		try {
			Session session = sessionFactory.getCurrentSession();
			LeadAttachment dbAttachment = (LeadAttachment) session.get(
					LeadAttachment.class, atttId);
			if (dbAttachment == null) {
				throw new ERPException(
						ErrorCodes.COULD_NOT_FIND_LEAD_ATTACHMENT,
						"Couldn't find leadAttachment with Id : " + atttId);
			}
			session.delete(dbAttachment);
			session.flush();
		} catch (ERPException e) {
			throw e;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_DELETING_LEAD_ATTACHMENT,
					"Error occurred while deleting lead attachment", e);
		}
	}

}
