package com.travel.erp.dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;
import org.hibernate.exception.ConstraintViolationException;
import org.hibernate.exception.DataException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.travel.erp.dao.LeadDao;
import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.model.Customer;
import com.travel.erp.model.Employee;
import com.travel.erp.model.Lead;
import com.travel.erp.model.LeadAttachment;
import com.travel.erp.model.LeadSource;
import com.travel.erp.model.LeadStatus;
import com.travel.erp.util.Utils;

@Repository("leadDao")
public class LeadDaoImpl implements LeadDao {

	@Autowired
	private SessionFactory sessionFactory;

	public LeadDaoImpl() {
	}

	public LeadDaoImpl(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Transactional
	public int createLead(Lead lead) {
		try {
			Date date = new Date();
			lead.setCreationTime(date);
			lead.setUpdateTime(date);
			sessionFactory.getCurrentSession().saveOrUpdate(lead);
			sessionFactory.getCurrentSession().flush();
			return lead.getId();
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("customer")) {
					throw new ERPException(
							ErrorCodes.INVALID_CUSTOMER_FOR_LEAD,
							"Invalid customer id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}
				if (message.contains("assignedTo")) {
					throw new ERPException(
							ErrorCodes.INVALID_ASSIGNEE_ID_FOR_LEAD,
							"Invalid assignee id provided", e);
				}
			}
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD, e
					.getSQLException().getMessage(), e);

		} catch (DataException e) {
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_CREATING_LEAD,
					"Error occurred while adding lead", e);
		}
	}

	public Lead getLeadInfo(int id) {
		Session session = sessionFactory.openSession();
		try {
			return (Lead) session.get(Lead.class, id);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_LEAD,
					"Couldn't find lead with leadId : " + id, e);
		} finally {
			session.close();
		}
	}

	@Override
	@Transactional
	public void updateLead(Lead lead) {
		try {
			Session session = sessionFactory.getCurrentSession();
			Lead oldLead = (Lead) session.get(Lead.class, lead.getId());
			if (oldLead == null) {
				throw new ERPException(ErrorCodes.COULD_NOT_FIND_LEAD,
						"Couldn't find lead with Id : " + lead.getId());
			}
			oldLead.setAdditionalInfo(StringUtils.isEmpty(lead
					.getAdditionalInfo()) ? oldLead.getAdditionalInfo() : lead
					.getAdditionalInfo());

			oldLead.setAdultCount(lead.getAdultCount() != null ? lead
					.getAdultCount() : oldLead.getAdultCount());

			oldLead.setAssignedTo(lead.getAssignedTo() != null ? lead
					.getAssignedTo() : oldLead.getAssignedTo());
			oldLead.setDepartureDate(lead.getDepartureDate() != null ? lead
					.getDepartureDate() : oldLead.getDepartureDate());
			oldLead.setDestinations(lead.getDestinations() != null ? lead
					.getDestinations() : oldLead.getDestinations());
			oldLead.setKidsCount(lead.getKidsCount() != null ? lead
					.getKidsCount() : oldLead.getKidsCount());
			oldLead.setLeadSource(lead.getLeadSource() != null ? lead
					.getLeadSource() : oldLead.getLeadSource());
			oldLead.setLeadStatus(lead.getLeadStatus() != null ? lead
					.getLeadStatus() : oldLead.getLeadStatus());
			oldLead.setLeavingFrom(lead.getLeavingFrom() != null ? lead
					.getLeavingFrom() : oldLead.getLeavingFrom());
			oldLead.setNights(lead.getNights() != null ? lead.getNights()
					: oldLead.getNights());
			Date date = new Date();
			oldLead.setUpdateTime(date);
			session.update(oldLead);
			session.flush();

		} catch (ERPException e) {
			throw e;
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("customer")) {
					throw new ERPException(
							ErrorCodes.INVALID_CUSTOMER_FOR_LEAD,
							"Invalid customer id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}
				if (message.contains("assignedTo")) {
					throw new ERPException(
							ErrorCodes.INVALID_ASSIGNEE_ID_FOR_LEAD,
							"Invalid assignee id provided", e);
				}

			}
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD, e
					.getSQLException().getMessage(), e);

		} catch (DataException e) {
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_LEAD,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_UPDATING_LEAD,
					"Error occurred while updating lead", e);
		}

	}

	@Override
	@Transactional
	public void deleteLead(int leadId) {
		try {
			Session session = sessionFactory.getCurrentSession();
			Lead lead = (Lead) session.get(Lead.class, leadId);
			if (lead == null) {
				throw new ERPException(ErrorCodes.COULD_NOT_FIND_LEAD,
						"Couldn't find lead with Id : " + leadId);
			}
			// TODO : Check for booking for this lead. If booking there dont
			// delete.
			Criteria query = session.createCriteria(LeadAttachment.class);
			query.add(Restrictions.eq("lead", lead));
			List<LeadAttachment> list = (List<LeadAttachment>) query.list();
			if (list != null) {
				for (LeadAttachment attachment : list) {
					session.delete(attachment);
				}
			}
			session.delete(lead);
			session.flush();
		} catch (ERPException e) {
			throw e;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_DELETING_LEAD,
					"Error occurred while deleting lead", e);
		}
	}

	@Override
	public List<Lead> searchLeads(Integer leadId, String emailId,
			String phoneNo, String destinations, Integer customerId,
			String departureDate, String leadSource, String leadStatus,
			Integer createdBy, Integer assignedTo, String fromDate,
			String toDate) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(Lead.class);
			if (leadId != null) {
				query.add(Restrictions.eq("id", leadId));
			}
			if (!StringUtils.isEmpty(emailId)) {
				Criteria custQuery = session.createCriteria(Customer.class);
				custQuery.add(Restrictions.like("emailId", emailId,
						MatchMode.START));
				List<Customer> list = (List<Customer>) custQuery.list();
				if (list != null) {
					query.add(Restrictions.in("customer", list));
				}
			}
			if (!StringUtils.isEmpty(phoneNo)) {
				Criteria custQuery = session.createCriteria(Customer.class);
				custQuery.add(Restrictions.like("phoneNo", phoneNo,
						MatchMode.START));
				List<Customer> list = (List<Customer>) custQuery.list();
				if (list != null) {
					query.add(Restrictions.in("customer", list));
				}
			}
			if (!StringUtils.isEmpty(destinations)) {
				query.add(Restrictions.like("destinations",
						destinations.toUpperCase(), MatchMode.ANYWHERE));
			}
			if (customerId != null) {
				Customer customer = new Customer();
				customer.setId(customerId);
				query.add(Restrictions.eq("customer", customer));
			}
			if (!StringUtils.isEmpty(departureDate)) {
				Date depDate = Utils.getDateFromString(departureDate);
				query.add(Restrictions.eq("departureDate", depDate));
			}
			if (!StringUtils.isEmpty(leadSource)) {
				query.add(Restrictions.eq("leadSource", leadSource));
			}
			if (!StringUtils.isEmpty(leadStatus)) {
				query.add(Restrictions.eq("leadStatus", leadStatus));
			}
			if (createdBy != null) {
				Employee createdByEmp = new Employee();
				createdByEmp.setId(createdBy);
				query.add(Restrictions.eq("createdBy", createdByEmp));
			}
			if (assignedTo != null) {
				Employee assignedToEmp = new Employee();
				assignedToEmp.setId(assignedTo);
				query.add(Restrictions.eq("assignedTo", assignedToEmp));
			}
			if (!StringUtils.isEmpty(fromDate)) {
				Date from = Utils.getDateFromString(fromDate);
				Date to = new Date();
				if (!StringUtils.isEmpty(toDate)) {
					to = Utils.getDateFromString(toDate);
				}
				query.add(Restrictions.between("creationTime", from, to));
			}
			List<Lead> list = (List<Lead>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR, e.getMessage());
		} finally {
			session.close();
		}

	}

	@Override
	public List<LeadStatus> getLeadStatusValues() {
		Session session = sessionFactory.openSession();
		try {
			List<LeadStatus> list = (List<LeadStatus>) session.createCriteria(
					LeadStatus.class).list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR, e.getMessage());
		} finally {
			session.close();
		}

	}

	@Override
	public List<LeadSource> getLeadSources() {
		Session session = sessionFactory.openSession();
		try {
			List<LeadSource> list = (List<LeadSource>) session.createCriteria(
					LeadSource.class).list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR, e.getMessage());
		} finally {
			session.close();
		}

	}
}
