package com.travel.erp.dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.hibernate.exception.ConstraintViolationException;
import org.hibernate.exception.DataException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.travel.erp.dao.NotesDao;
import com.travel.erp.exception.ERPException;
import com.travel.erp.exception.ErrorCodes;
import com.travel.erp.model.Lead;
import com.travel.erp.model.LeadAttachment;
import com.travel.erp.model.Note;

@Repository("notesDao")
public class NotesDaoImpl implements NotesDao {
	@Autowired
	private SessionFactory sessionFactory;

	public NotesDaoImpl() {
	}

	public NotesDaoImpl(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Transactional
	public int addNotes(Note note) {
		try {
			Date date = new Date();
			note.setUpdatedAt(date);
			note.setCreatedAt(date);
			sessionFactory.getCurrentSession().save(note);
			sessionFactory.getCurrentSession().flush();
			return note.getNoteId();
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("leadId")) {
					throw new ERPException(ErrorCodes.INVALID_LEAD_ID,
							"Invalid lead id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}

			}
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_NOTES, e
					.getSQLException().getMessage(), e);

		} catch (DataException e) {
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_NOTES,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error occurred while adding notes", e);
		}
	}

	@Override
	public Note getNoteInfo(Integer noteId) {
		Session session = sessionFactory.openSession();
		try {
			return (Note) session.get(Note.class, noteId);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.COULD_NOT_FIND_NOTE,
					"Couldn't find note with noteId : " + noteId, e);
		} finally {
			session.close();
		}
	}

	@Override
	public List<Note> getNotesOfALead(Integer leadId) {
		Session session = sessionFactory.openSession();
		try {
			Criteria query = session.createCriteria(Note.class);
			query.add(Restrictions.eq("leadId", leadId));
			List<Note> list = (List<Note>) query.list();
			return list;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.UNKNOWN_ERROR,
					"Error while getting notes for leadId : " + leadId);
		} finally {
			session.close();
		}
	}

	@Override
	@Transactional
	public void updateNote(Note note) {
		try {
			Session session = sessionFactory.getCurrentSession();
			Note oldNote = (Note) session.get(Note.class, note.getNoteId());
			if (oldNote == null) {
				throw new ERPException(ErrorCodes.COULD_NOT_FIND_NOTE,
						"Couldn't find note with Id : " + note.getNoteId());
			}
			oldNote.setNotes(StringUtils.isEmpty(note.getNotes()) ? oldNote
					.getNotes() : note.getNotes());

			oldNote.setNotesSource(StringUtils.isEmpty(note.getNotesSource()) ? oldNote
					.getNotesSource() : note.getNotesSource());
			oldNote.setUpdatedAt(new Date());
			session.update(oldNote);
			session.flush();

		} catch (ERPException e) {
			throw e;
		} catch (ConstraintViolationException e) {
			String message = e.getSQLException().getMessage();
			if (message.contains("foreign key constraint fails")) {
				if (message.contains("leadId")) {
					throw new ERPException(ErrorCodes.INVALID_LEAD_ID,
							"Invalid lead id provided", e);
				}
				if (message.contains("createdBy")) {
					throw new ERPException(
							ErrorCodes.INVALID_EMPLOYEE_ID_FOR_LEAD,
							"Invalid employee id provided", e);
				}

			}
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_NOTES, e
					.getSQLException().getMessage(), e);
		} catch (DataException e) {
			throw new ERPException(ErrorCodes.INVALID_DATA_FORMAT_FOR_NOTES,
					"Invalid input provided", e);
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_UPDATING_NOTE,
					"Error occurred while updating lead", e);
		}
	}

	@Override
	@Transactional
	public void deleteNote(Integer noteId) {
		try {
			Session session = sessionFactory.getCurrentSession();
			Note note = (Note) session.get(Note.class, noteId);
			if (note == null) {
				throw new ERPException(ErrorCodes.COULD_NOT_FIND_NOTE,
						"Couldn't find note with Id : " + noteId);
			}

			session.delete(note);
			session.flush();
		} catch (ERPException e) {
			throw e;
		} catch (Exception e) {
			throw new ERPException(ErrorCodes.FAILURE_DELETING_NOTE,
					"Error occurred while deleting note", e);
		}

	}

}
